window.Gleam.initSearch([{"doc":"bison","title":"bison","content":"![bison](https://raw.githubusercontent.com/massivefermion/bison/main/banner.png)\n\n[![Package Version](https://img.shields.io/hexpm/v/bison)](https://hex.pm/packages/bison)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/bison/)\n\n# bison (formerly gleam_bson)\n\nbson encoder and decoder for gleam\n\n## &lt;img width=32 src=&quot;https://raw.githubusercontent.com/massivefermion/bison/main/icon.png&quot;&gt; Quick start\n\n```sh\ngleam test  # Run the tests\ngleam shell # Run an Erlang shell\n```\n\n## &lt;img width=32 src=&quot;https://raw.githubusercontent.com/massivefermion/bison/main/icon.png&quot;&gt; Installation\n\n```sh\ngleam add bison\n```\n\n## &lt;img width=32 src=&quot;https://raw.githubusercontent.com/massivefermion/bison/main/icon.png&quot;&gt; Roadmap\n\n- [x] support encoding and decoding basic bson types (null, string, int32, int64, double, boolean, objectId, array, document).\n- [x] support encoding and decoding min, max, timestamp, datetime, javascript and regex bson types.\n- [x] support encoding and decoding the generic, md5, uuid and user-defined binary subtypes.\n- [x] support generating new object-ids\n- [ ] support encoding and decoding decimal128 bson type.\n- [ ] support encoding and decoding encrypted binary subtype.\n\n## &lt;img width=32 src=&quot;https://raw.githubusercontent.com/massivefermion/bison/main/icon.png&quot;&gt; Usage\n\n### Encoding\n\n```gleam\nimport gleam/list\nimport gleam/result\nimport bison/md5\nimport bison/bson\nimport bison.{encode}\nimport bison/object_id\n\nfn calf_to_bson(calf: Calf) -&gt; Result(BitArray, Nil) {\n  use id &lt;- result.then(object_id.from_string(calf.id))\n  use checksum &lt;- result.then(md5.from_string(calf.checksum))\n\n  Ok(encode([\n    #(&quot;id&quot;, bson.ObjectId(id)),\n    #(&quot;age&quot;, bson.Int32(calf.age)),\n    #(&quot;name&quot;, bson.String(calf.name)),\n    #(&quot;weight&quot;, bson.Double(calf.weight)),\n    #(&quot;birthdate&quot;, bson.DateTime(calf.birthdate)),\n    #(&quot;is_healthy&quot;, bson.Boolean(calf.is_healthy)),\n    #(&quot;checksum&quot;, bson.Binary(bson.MD5(checksum))),\n    #(&quot;nicknames&quot;, bson.Array(list.map(calf.nicknames, bson.String))),\n  ]))\n}\n```\n\n### Decoding\n\n```gleam\nimport gleam/list\nimport gleam/result\nimport bison/md5\nimport bison/bson\nimport bison.{decode}\nimport bison/object_id\n\nfn calf_from_bson(data: BitArray) -&gt; Result(Calf, Nil) {\n  use doc &lt;- result.then(decode(data))\n\n  let [\n    #(&quot;id&quot;, bson.ObjectId(id)),\n    #(&quot;age&quot;, bson.Int32(age)),\n    #(&quot;name&quot;, bson.String(name)),\n    #(&quot;weight&quot;, bson.Double(weight)),\n    #(&quot;nicknames&quot;, bson.Array(nicknames)),\n    #(&quot;birthdate&quot;, bson.DateTime(birthdate)),\n    #(&quot;is_healthy&quot;, bson.Boolean(is_healthy)),\n    #(&quot;checksum&quot;, bson.Binary(bson.MD5(checksum))),\n  ] = doc\n\n  Ok(Calf(\n    id: object_id.to_string(id),\n    age: age,\n    name: name,\n    weight: weight,\n    nicknames: list.map(\n      nicknames,\n      fn(n) {\n        let assert bson.String(n) = n\n        n\n      },\n    ),\n    birthdate: birthdate,\n    is_healthy: is_healthy,\n    checksum: md5.to_string(checksum),\n  ))\n}\n\n```\n","url":"index.html"},{"doc":"bison/md5","title":"MD5","content":"pub opaque type MD5\n\n\nSynonyms:\nbison/md5.MD5\nbison/md5 MD5","url":"bison/md5.html#MD5"},{"doc":"bison/md5","title":"from_bit_array","content":"pub fn from_bit_array(md5: BitArray) -&gt; Result(MD5, Nil)\n\nSynonyms:\nbison/md5.from_bit_array\nbison/md5 from_bit_array","url":"bison/md5.html#from_bit_array"},{"doc":"bison/md5","title":"from_int_list","content":"pub fn from_int_list(md5: List(Int)) -&gt; Result(MD5, Nil)\n\nSynonyms:\nbison/md5.from_int_list\nbison/md5 from_int_list","url":"bison/md5.html#from_int_list"},{"doc":"bison/md5","title":"from_string","content":"pub fn from_string(md5: String) -&gt; Result(MD5, Nil)\n\nSynonyms:\nbison/md5.from_string\nbison/md5 from_string","url":"bison/md5.html#from_string"},{"doc":"bison/md5","title":"to_bit_array","content":"pub fn to_bit_array(md5: MD5) -&gt; BitArray\n\nSynonyms:\nbison/md5.to_bit_array\nbison/md5 to_bit_array","url":"bison/md5.html#to_bit_array"},{"doc":"bison/md5","title":"to_int_list","content":"pub fn to_int_list(md5: MD5) -&gt; List(Int)\n\nSynonyms:\nbison/md5.to_int_list\nbison/md5 to_int_list","url":"bison/md5.html#to_int_list"},{"doc":"bison/md5","title":"to_string","content":"pub fn to_string(md5: MD5) -&gt; String\n\nSynonyms:\nbison/md5.to_string\nbison/md5 to_string","url":"bison/md5.html#to_string"},{"doc":"bison/md5","title":"bison/md5","content":"","url":"bison/md5.html"},{"doc":"bison/uuid","title":"UUID","content":"pub opaque type UUID\n\n\nSynonyms:\nbison/uuid.UUID\nbison/uuid UUID","url":"bison/uuid.html#UUID"},{"doc":"bison/uuid","title":"from_bit_array","content":"pub fn from_bit_array(uuid: BitArray) -&gt; Result(UUID, Nil)\n\nSynonyms:\nbison/uuid.from_bit_array\nbison/uuid from_bit_array","url":"bison/uuid.html#from_bit_array"},{"doc":"bison/uuid","title":"from_int_list","content":"pub fn from_int_list(uuid: List(Int)) -&gt; Result(UUID, Nil)\n\nSynonyms:\nbison/uuid.from_int_list\nbison/uuid from_int_list","url":"bison/uuid.html#from_int_list"},{"doc":"bison/uuid","title":"from_string","content":"pub fn from_string(uuid: String) -&gt; Result(UUID, Nil)\n\nSynonyms:\nbison/uuid.from_string\nbison/uuid from_string","url":"bison/uuid.html#from_string"},{"doc":"bison/uuid","title":"to_bit_array","content":"pub fn to_bit_array(uuid: UUID) -&gt; BitArray\n\nSynonyms:\nbison/uuid.to_bit_array\nbison/uuid to_bit_array","url":"bison/uuid.html#to_bit_array"},{"doc":"bison/uuid","title":"to_int_list","content":"pub fn to_int_list(uuid: UUID) -&gt; List(Int)\n\nSynonyms:\nbison/uuid.to_int_list\nbison/uuid to_int_list","url":"bison/uuid.html#to_int_list"},{"doc":"bison/uuid","title":"to_string","content":"pub fn to_string(uuid: UUID) -&gt; String\n\nSynonyms:\nbison/uuid.to_string\nbison/uuid to_string","url":"bison/uuid.html#to_string"},{"doc":"bison/uuid","title":"bison/uuid","content":"","url":"bison/uuid.html"},{"doc":"bison/custom","title":"Custom","content":"pub opaque type Custom\n\n\nSynonyms:\nbison/custom.Custom\nbison/custom Custom","url":"bison/custom.html#Custom"},{"doc":"bison/custom","title":"max_binary_custom_code","content":"pub const max_binary_custom_code: Int = 0xff\n\nSynonyms:\nbison/custom.max_binary_custom_code\nbison/custom max_binary_custom_code","url":"bison/custom.html#max_binary_custom_code"},{"doc":"bison/custom","title":"min_binary_custom_code","content":"pub const min_binary_custom_code: Int = 0x80\n\nSynonyms:\nbison/custom.min_binary_custom_code\nbison/custom min_binary_custom_code","url":"bison/custom.html#min_binary_custom_code"},{"doc":"bison/custom","title":"from_bit_array_with_code","content":"pub fn from_bit_array_with_code(\n  code: Int,\n  value: BitArray,\n) -&gt; Result(Custom, Nil)\n\nSynonyms:\nbison/custom.from_bit_array_with_code\nbison/custom from_bit_array_with_code","url":"bison/custom.html#from_bit_array_with_code"},{"doc":"bison/custom","title":"to_bit_array_with_code","content":"pub fn to_bit_array_with_code(custom: Custom) -&gt; #(Int, BitArray)\n\nSynonyms:\nbison/custom.to_bit_array_with_code\nbison/custom to_bit_array_with_code","url":"bison/custom.html#to_bit_array_with_code"},{"doc":"bison/custom","title":"bison/custom","content":"","url":"bison/custom.html"},{"doc":"bison/generic","title":"Generic","content":"pub opaque type Generic\n\n\nSynonyms:\nbison/generic.Generic\nbison/generic Generic","url":"bison/generic.html#Generic"},{"doc":"bison/generic","title":"from_bit_array","content":"pub fn from_bit_array(data: BitArray) -&gt; Result(Generic, Nil)\n\nSynonyms:\nbison/generic.from_bit_array\nbison/generic from_bit_array","url":"bison/generic.html#from_bit_array"},{"doc":"bison/generic","title":"from_int_list","content":"pub fn from_int_list(data: List(Int)) -&gt; Generic\n\nSynonyms:\nbison/generic.from_int_list\nbison/generic from_int_list","url":"bison/generic.html#from_int_list"},{"doc":"bison/generic","title":"from_string","content":"pub fn from_string(data: String) -&gt; Generic\n\nSynonyms:\nbison/generic.from_string\nbison/generic from_string","url":"bison/generic.html#from_string"},{"doc":"bison/generic","title":"to_bit_array","content":"pub fn to_bit_array(generic: Generic) -&gt; BitArray\n\nSynonyms:\nbison/generic.to_bit_array\nbison/generic to_bit_array","url":"bison/generic.html#to_bit_array"},{"doc":"bison/generic","title":"to_int_list","content":"pub fn to_int_list(generic: Generic) -&gt; List(Int)\n\nSynonyms:\nbison/generic.to_int_list\nbison/generic to_int_list","url":"bison/generic.html#to_int_list"},{"doc":"bison/generic","title":"to_string","content":"pub fn to_string(generic: Generic) -&gt; Result(String, Nil)\n\nSynonyms:\nbison/generic.to_string\nbison/generic to_string","url":"bison/generic.html#to_string"},{"doc":"bison/generic","title":"bison/generic","content":"","url":"bison/generic.html"},{"doc":"bison/object_id","title":"ObjectId","content":"pub opaque type ObjectId\n\n\nSynonyms:\nbison/object_id.ObjectId\nbison/object_id ObjectId","url":"bison/object_id.html#ObjectId"},{"doc":"bison/object_id","title":"compare","content":"pub fn compare(a: ObjectId, b: ObjectId) -&gt; Order\n\nSynonyms:\nbison/object_id.compare\nbison/object_id compare","url":"bison/object_id.html#compare"},{"doc":"bison/object_id","title":"from_bit_array","content":"pub fn from_bit_array(id: BitArray) -&gt; Result(ObjectId, Nil)\n\nSynonyms:\nbison/object_id.from_bit_array\nbison/object_id from_bit_array","url":"bison/object_id.html#from_bit_array"},{"doc":"bison/object_id","title":"from_datetime","content":"pub fn from_datetime(datetime: Time) -&gt; ObjectId\n see [birl](https://hex.pm/packages/birl)!\nSynonyms:\nbison/object_id.from_datetime\nbison/object_id from_datetime","url":"bison/object_id.html#from_datetime"},{"doc":"bison/object_id","title":"from_int_list","content":"pub fn from_int_list(id: List(Int)) -&gt; Result(ObjectId, Nil)\n\nSynonyms:\nbison/object_id.from_int_list\nbison/object_id from_int_list","url":"bison/object_id.html#from_int_list"},{"doc":"bison/object_id","title":"from_string","content":"pub fn from_string(id: String) -&gt; Result(ObjectId, Nil)\n\nSynonyms:\nbison/object_id.from_string\nbison/object_id from_string","url":"bison/object_id.html#from_string"},{"doc":"bison/object_id","title":"new","content":"pub fn new() -&gt; ObjectId\n\nSynonyms:\nbison/object_id.new\nbison/object_id new","url":"bison/object_id.html#new"},{"doc":"bison/object_id","title":"range","content":"pub fn range(\n  from a: ObjectId,\n  to b: Option(ObjectId),\n  step s: Duration,\n) -&gt; Iterator(ObjectId)\n can be used to create a time range starting from time `a` with step `s`,\n which is of type `duration.Duration` from package [birl](https://hex.pm/packages/birl)\n\n if `b` is `option.None` the range will be infinite\nSynonyms:\nbison/object_id.range\nbison/object_id range","url":"bison/object_id.html#range"},{"doc":"bison/object_id","title":"to_bit_array","content":"pub fn to_bit_array(id: ObjectId) -&gt; BitArray\n\nSynonyms:\nbison/object_id.to_bit_array\nbison/object_id to_bit_array","url":"bison/object_id.html#to_bit_array"},{"doc":"bison/object_id","title":"to_datetime","content":"pub fn to_datetime(id: ObjectId) -&gt; Time\n see [birl](https://hex.pm/packages/birl)!\nSynonyms:\nbison/object_id.to_datetime\nbison/object_id to_datetime","url":"bison/object_id.html#to_datetime"},{"doc":"bison/object_id","title":"to_int_list","content":"pub fn to_int_list(id: ObjectId) -&gt; List(Int)\n\nSynonyms:\nbison/object_id.to_int_list\nbison/object_id to_int_list","url":"bison/object_id.html#to_int_list"},{"doc":"bison/object_id","title":"to_string","content":"pub fn to_string(id: ObjectId) -&gt; String\n\nSynonyms:\nbison/object_id.to_string\nbison/object_id to_string","url":"bison/object_id.html#to_string"},{"doc":"bison/object_id","title":"bison/object_id","content":"","url":"bison/object_id.html"},{"doc":"bison/bson","title":"Binary","content":"pub type Binary {\n  MD5(MD5)\n  UUID(UUID)\n  Custom(Custom)\n  Generic(Generic)\n}\n\nMD5(MD5)\n\n\nUUID(UUID)\n\n\nCustom(Custom)\n\n\nGeneric(Generic)\n\n\nSynonyms:\nbison/bson.Binary\nbison/bson Binary","url":"bison/bson.html#Binary"},{"doc":"bison/bson","title":"Value","content":"pub type Value {\n  Min\n  Max\n  NaN\n  Null\n  Infinity\n  JS(String)\n  Int32(Int)\n  Int64(Int)\n  Double(Float)\n  Boolean(Bool)\n  Binary(Binary)\n  String(String)\n  DateTime(Time)\n  NegativeInfinity\n  Array(List(Value))\n  ObjectId(ObjectId)\n  Timestamp(Int, Int)\n  Regex(String, String)\n  Document(List(#(String, Value)))\n}\n if you&#39;re not familiar with type `DateTime`, see [birl](https://hex.pm/packages/birl)!\nMin\n\n\nMax\n\n\nNaN\n\n\nNull\n\n\nInfinity\n\n\nJS(String)\n\n\nInt32(Int)\n\n\nInt64(Int)\n\n\nDouble(Float)\n\n\nBoolean(Bool)\n\n\nBinary(Binary)\n\n\nString(String)\n\n\nDateTime(Time)\n\n\nNegativeInfinity\n\n\nArray(List(Value))\n\n\nObjectId(ObjectId)\n\n\nTimestamp(Int, Int)\n\n\nRegex(String, String)\n\n\nDocument(List(#(String, Value)))\n\n\nSynonyms:\nbison/bson.Value\nbison/bson Value","url":"bison/bson.html#Value"},{"doc":"bison/bson","title":"bison/bson","content":"","url":"bison/bson.html"},{"doc":"bison/ejson","title":"from_canonical","content":"pub fn from_canonical(\n  doc: String,\n) -&gt; Result(List(#(String, Value)), DecodeError)\n\nSynonyms:\nbison/ejson.from_canonical\nbison/ejson from_canonical","url":"bison/ejson.html#from_canonical"},{"doc":"bison/ejson","title":"to_canonical","content":"pub fn to_canonical(doc: List(#(String, Value))) -&gt; String\n\nSynonyms:\nbison/ejson.to_canonical\nbison/ejson to_canonical","url":"bison/ejson.html#to_canonical"},{"doc":"bison/ejson","title":"bison/ejson","content":" For more information see [here](https://www.mongodb.com/docs/manual/reference/mongodb-extended-json)!","url":"bison/ejson.html"},{"doc":"bison","title":"decode","content":"pub fn decode(\n  data: BitArray,\n) -&gt; Result(List(#(String, Value)), Nil)\n\nSynonyms:\nbison.decode\nbison decode","url":"bison.html#decode"},{"doc":"bison","title":"encode","content":"pub fn encode(doc: List(#(String, Value))) -&gt; BitArray\n\nSynonyms:\nbison.encode\nbison encode","url":"bison.html#encode"},{"doc":"bison","title":"bison","content":"","url":"bison.html"}]);